==> turbina-2.01eta.tgz: ver2.01eta/svexchange.cpp <==
#include <stdio.h>
#include "svexchange.h"
#include "trbsocket.h"
#include "names.h"
#include "success.h"
#include "trberror.h"

using namespace std;

namespace SVexchange {
	status _status = READY;
	status _erstatus = READY;
	string _newObjectID = "";
	string _data = "";
}

int readInstruction(string& from, int& comid, string& keyword,
							string& param, string& paramData) {
	comid = -1;
	keyword = "";
	param = "";
	paramData = "";

	string::size_type i_comid = from.find(" ",0);
	if(i_comid == string::npos) return FAIL;
	string comid_str = from.substr(0,i_comid);
	comid = atoi(comid_str.c_str());
	if(comid == -1) return FAIL;

	string::size_type i1 = from.find(" ", i_comid+1);
	if(i1 == string::npos) {
		keyword = from.substr(i_comid+1);
		return SUCCESS;
	}
	keyword = from.substr(i_comid+1,i1-i_comid-1);
	string::size_type i2 = from.find("=",i1+1);
	if(i2 == string::npos) {
		i2 = from.find(" ",i1+1);
		if(i2 == string::npos) {
			param = from.substr(i1+1);
			return SUCCESS;
		}
	}
	param = from.substr(i1+1,i2-i1-1);
	string::size_type i3 = from.find(" ",i2+1);
	if(i3 == string::npos) {
		paramData = from.substr(i2+1);
	} else {
		paramData = from.substr(i2+1,i3-i2-1);
	}
	return SUCCESS;
}

SVexchange::instruction SVexchange::translate(string& what, int& comid) {
	string keyword = "";
	string param = "";
	string paramData = "";
	if(readInstruction(what, comid, keyword, param, paramData) == FAIL) {
		setErstatus(ERSYN);
		sendError(comid);
		return UNKNOWN;
	}

	try {

		if((keyword == "GET") || (keyword == "get")) {
			if((param == "IDENT") || (param == "ident")) return IDENT;
			if((param == "STATUS") || (param == "status")) return STATUS;
			if((param == "DATA") || (param == "data")) return DATA;
			throw SVError("");
		}
		if((keyword == "SET") || (keyword == "set")) {
			if((param == "OBJECT") || (param == "object")) {
				if(paramData != "") {
					_newObjectID = paramData;
					return NEW_OBJECT;
				}
			}
			throw SVError("");
		}
		if((keyword == "INIT") || (keyword == "init")) return INIT;
		if((keyword == "PARK") || (keyword == "park")) return PARK;
		if((keyword == "RUN") || (keyword == "run")) {
			if(param == "") return RUN;
			if((param == "BKGR") || (param == "bkgr")) return RUN_BKGR;
			throw SVError("");
		}
		if((keyword == "STOP") || (keyword == "stop")) {
			if((param == "NOW") || (param == "now")) return STOP_NOW;
			throw SVError("");
		}
		if((keyword == "QUIT") || (keyword == "quit")) return QUIT;
		throw SVError("");
	} catch (SVError) {
		setErstatus(ERSYN);
		sendError(comid);
		return UNKNOWN;
	}
}

const char* SVexchange::getNewObject(void) {
	return _newObjectID.c_str();
}

void SVexchange::setStatus(status stat) {
	_status = stat;
}

void SVexchange::setErstatus(status erstatus) {
	_erstatus = erstatus;
}

void SVexchange::setData(const char* data) {
	_data = data;
}

const char* typeStatus(SVexchange::status stat) {
	switch(stat) {
		case SVexchange::READY	: return "READY";
		case SVexchange::PARKED	: return "PARKED";
		case SVexchange::BUSY	: return "BUSY";
		case SVexchange::ERSYN	: return "ERSYN";
		case SVexchange::ERFAT	: return "ERFAT";
		case SVexchange::ERANG	: return "ERANG";
		case SVexchange::NOSTAR	: return "NOSTAR";
		default		: return "UNKNOWN";
	}
}

char toSV[80];

int SVexchange::sendOK(int comid,int waitsec) {
	if(waitsec) {
		sprintf(toSV,"OK status=%s wait=%d\n",typeStatus(_status),waitsec);
	} else {
		sprintf(toSV,"OK status=%s\n",typeStatus(_status));
	}
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendIdent(int comid) {
	sprintf(toSV,"OK ident=\"TURBINA V%s\"\n",VERSION);
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendStatus(int comid) {
	sprintf(toSV,"OK status=%s\n",typeStatus(_status));
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendData(int comid) {
	sprintf(toSV,"OK DATA=\"bkgr=%s\"\n",_data.c_str());
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendError(int comid) {
	sprintf(toSV,"ERROR status=%s\n",typeStatus(_erstatus));
	return trbsocket::sendToSV(comid, toSV);
}

==> turbina-2.02beta.tgz: ver2.02beta/svexchange.cpp <==
#include <stdio.h>
#include "svexchange.h"
#include "trbsocket.h"
#include "names.h"
#include "success.h"
#include "trberror.h"

using namespace std;

namespace SVexchange {
	status _status = READY;
	status _erstatus = READY;
//	string _newObjectID = "";
	string _data = "";
}

int readInstruction(string& from, int& comid, string& keyword,
							string& param, string& paramData) {
	comid = -1;
	keyword = "";
	param = "";
	paramData = "";

	string::size_type i_comid = from.find(" ",0);
	if(i_comid == string::npos) return FAIL;
	string comid_str = from.substr(0,i_comid);
	comid = atoi(comid_str.c_str());
	if(comid == -1) return FAIL;

	string::size_type i1 = from.find(" ", i_comid+1);
	if(i1 == string::npos) {
		keyword = from.substr(i_comid+1);
		return SUCCESS;
	}
	keyword = from.substr(i_comid+1,i1-i_comid-1);
	string::size_type i2 = from.find("=",i1+1);
	if(i2 == string::npos) {
		i2 = from.find(" ",i1+1);
		if(i2 == string::npos) {
			param = from.substr(i1+1);
			return SUCCESS;
		}
	}
	param = from.substr(i1+1,i2-i1-1);
	string::size_type i3 = from.find(" ",i2+1);
	if(i3 == string::npos) {
		paramData = from.substr(i2+1);
	} else {
		paramData = from.substr(i2+1,i3-i2-1);
	}
	return SUCCESS;
}

int SVexchange::translate(std::string& what, Instruction& instruct) {
	string command = "";
	string param = "";
	string value = "";
	int comid;
	if(readInstruction(what, comid, command, param, value) == FAIL) {
		setErstatus(ERSYN);
		sendError(comid);
		return FAIL;
	}

	instruct.comid = comid;

	if((command == "GET") || (command == "get")) instruct.command = GET;
	else if((command == "SET") || (command == "set")) instruct.command = SET;
	else if((command == "INIT") || (command == "init")) instruct.command = INIT;
	else if((command == "PARK") || (command == "park")) instruct.command = PARK;
	else if((command == "RUN") || (command == "run")) instruct.command = RUN;
	else if((command == "STOP") || (command == "stop")) instruct.command = STOP;
	else instruct.command = UNKNOWN_COMMAND;

	if((param == "IDENT") || (param == "ident")) instruct.param = IDENT;
	else if((param == "STATUS") || (param == "status")) instruct.param = STATUS;
	else if((param == "DATA") || (param == "data")) instruct.param = DATA;
	else if((param == "OBJECT") || (param == "object")) instruct.param = OBJECT;
	else if((param == "BKGR") || (param == "bkgr")) instruct.param = BKGR;
	else if((param == "FLUX") || (param == "flux")) instruct.param = FLUX;
	else if((param == "NOW") || (param == "now")) instruct.param = NOW;
	else if(param == "") instruct.param = NOTHING;
	else instruct.param = UNKNOWN_PARAM;
	
	instruct.value = value;
	return SUCCESS;
}

void SVexchange::setStatus(status stat) {
	_status = stat;
}

void SVexchange::setErstatus(status erstatus) {
	_erstatus = erstatus;
}

void SVexchange::setData(const char* data) {
	_data = data;
}

const char* typeStatus(SVexchange::status stat) {
	switch(stat) {
		case SVexchange::READY	: return "READY";
		case SVexchange::PARKED	: return "PARKED";
		case SVexchange::BUSY	: return "BUSY";
		case SVexchange::ERSYN	: return "ERSYN";
		case SVexchange::ERFAT	: return "ERFAT";
		case SVexchange::ERANG	: return "ERANG";
		case SVexchange::NOSTAR	: return "NOSTAR";
		default		: return "UNKNOWN";
	}
}

char toSV[80];

int SVexchange::sendOK(int comid,int waitsec) {
	if(waitsec) {
		sprintf(toSV,"OK status=%s wait=%d\n",typeStatus(_status),waitsec);
	} else {
		sprintf(toSV,"OK status=%s\n",typeStatus(_status));
	}
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendOK(int comid, std::string& result) {
	sprintf(toSV,"OK %s status=%s\n",result.c_str(), typeStatus(_status));
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendIdent(int comid) {
	sprintf(toSV,"OK ident=\"TURBINA V%s\"\n",VERSION);
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendStatus(int comid) {
	sprintf(toSV,"OK status=%s\n",typeStatus(_status));
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendData(int comid) {
	sprintf(toSV,"OK DATA=\"%s\"\n",_data.c_str());
	return trbsocket::sendToSV(comid, toSV);
}

int SVexchange::sendError(int comid) {
	sprintf(toSV,"ERROR status=%s\n",typeStatus(_erstatus));
	return trbsocket::sendToSV(comid, toSV);
}
